﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import {
  RacingSansOne_400Regular
} from '@expo-google-fonts/racing-sans-one';
import { NavigationContainer } from '@react-navigation/native';
import { MenuProvider } from 'react-native-popup-menu'
import axios, { AxiosRequestConfig } from 'axios';
import { useFonts } from 'expo-font';
import { getItemAsync } from 'expo-secure-store';
import { observer } from 'mobx-react';
import React, { useEffect, useRef, useState } from 'react';
import { AppState, SafeAreaView, StatusBar, Platform, Alert } from 'react-native';
import { QueryClient, QueryClientProvider } from 'react-query';
import { Splash } from '../components/Splash';
import { colors, SECURESTORE_ACCESSTOKEN, SECURESTORE_REFRESHTOKEN } from '../utils/GlobalUtils';
import { useAuth } from '../hooks/useAuth';
import AuthorizedNavigator from '../navigation/AuthorizedNavigator';
import UnauthorizedNavigator from '../navigation/UnauthorizedNavigator';
import { authStore } from '../libs/auth/AuthStore';
import { API_URL } from '../utils/RequestUtils';
import {reportAxiosError} from '../libs/auth/core'
import * as Updates from 'expo-updates'
import { Manifest, UpdateCheckResult } from 'expo-updates';

export enum TokenState {
  CheckingToken,
  ValidToken,
  InvalidToken
}

const queryClient = new QueryClient();

/**
 * Sub-root component of the app. Contains all global providers (NavigationContainer and SafeAreaProvider for React Navigation, QueryClientProvider for react-query) and is responsible for restricting unauthenticated users to the Intro screen by listening to {@link AuthStore}'s accessToken value.
 * @component
 */
const Atlas : React.FC = () => {
  /**
   * Flag that is switched on when the app is checking for tokens in the keystore and in memory. When true, "Logging you in.." and a spinner will be displayed to the user.
   */
  const [checkingToken, setCheckingToken] = useState<boolean>(true);
  const { refreshAccessToken } = useAuth();
  const updateDismissed = useRef<boolean>(false)
  const [fontsLoaded, error] = useFonts({
    RacingSansOne_400Regular
  });

    useEffect(() => {
      if (!__DEV__) {
        const timer = setInterval(async () => {
          const update = await Updates.checkForUpdateAsync()
          if (update.isAvailable && !updateDismissed.current) {
            updateDismissed.current = true
            setTimeout(() => {
              Alert.alert('Update Available', 'An update is available. Would you like to update now?', [
                {"text": "Yes", "onPress": async () => {
                  await Updates.fetchUpdateAsync()
                  await Updates.reloadAsync()
                }},
                {"text": "No", "onPress": () => {
                  Alert.alert('Update Available', 'Update dismissed, you can always revisit it in settings', [
                    {"text": "OK"}
                  ])
                }}
              ])
            }, 1000);
          }
      }, 5000)
      return () => clearInterval(timer)
    }}, [])

  /**
   * Checks if there is an access token available in {@link AuthStore}, then checks if that access token is valid by calling the API. 
   * If the response is valid, the access token will be stored in memory, otherwise the user will be directed to intro screen.
   */
  const checkToken = async () => {
    // check both the mobx store and secure storage for the token
    console.log('[Authentication]: Checking for access token in memory...')
    let currentAccessToken = authStore.accessToken;
    if (!currentAccessToken) {
      console.log('[Authentication]: No access token in memory, checking in secure store...')
      currentAccessToken = await getItemAsync(SECURESTORE_ACCESSTOKEN);
    }

    if (!currentAccessToken) {
      console.log('[Authentication]: No access token in secure store, attempting to use a refresh token...')
      let refreshToken  = authStore.refreshToken;
      if (!refreshToken) {
        refreshToken = await getItemAsync(SECURESTORE_REFRESHTOKEN)
      }

      if (refreshToken) {
        await refreshAccessToken()
        currentAccessToken = authStore.accessToken
      }
    }  
    
    if (currentAccessToken) {
      console.log('[Authentication]: Found access token, testing its validity...')
      // check to see if the token is valid by making test call
      const requestConfig: AxiosRequestConfig = {
        method: 'GET',
        url: API_URL + "/api/me/",
        headers: { "Authorization": "Bearer " + currentAccessToken }
      };

      try {
        const response = await axios(requestConfig);
        if (response.status == 200) {
          await authStore.setAccessTokenAsync(currentAccessToken);
          await authStore.setRefreshTokenAsync(await getItemAsync(SECURESTORE_REFRESHTOKEN))
          await authStore.setIdAsync(response.data.id)
          console.log('[Authentication]: Access token valid.')
        }
      } catch (error) {
        // check if access token can be refreshed
        console.log(error)
        if (error.response.status == 401) {
          try {
            await refreshAccessToken();
            // update authorization header w/ new token
            await axios({...requestConfig, headers: { "Authorization": "Bearer " + authStore.accessToken }}); 
          } catch (error) {
            
          }
        }
        // something went wrong with the api call, log error and clear auth state
        reportAxiosError('[Authentication]: Something went wrong when retrieving an access token', error)
        await authStore.setAccessTokenAsync(null);
        await authStore.setRefreshTokenAsync(null);
        await authStore.setNotificationTokenAsync(null);
        await authStore.setIdAsync(null);
      }
    }
    else {
      // no access token was found, user will be taken to login
      console.log('[Authentication]: No access token was found, prompting user to login.')
      await authStore.setAccessTokenAsync(null);
      await authStore.setRefreshTokenAsync(null);
      await authStore.setNotificationTokenAsync(null);
      await authStore.setIdAsync(null);
    }

    setCheckingToken(false);
  }
  
  useEffect(() => {
    /**
     * useEffect hook that is responsible for registering an appState "change" handler that will call {@linkcode checkToken} each time the app is opened or closed on the device.
     * @memberOf Atlas
     */
    function registerAppStateChangeHandler() {
      AppState.addEventListener("change", (appState: string) => {
        if (appState == 'active') {
          console.log('[Authentication]: App opened, checking auth tokens...')
          checkToken(); 
        }
      });
      return () => {
        AppState.removeEventListener("change", (appState: string) => {
          if (appState == 'active') {
            checkToken(); 
          }
        });
      };
    }
    registerAppStateChangeHandler();
  }, [AppState.currentState]);

  useEffect(() => {
    /**
   * Calls {@linkcode checkToken} when a change to the access token stored in {@link AuthStore} is detected. 
   * @memberOf Atlas
   */
    const checkTokenOnAccessTokenChange = async () => {
      console.log('[Authentication]: Change to accessToken detected, checking token state...')
      await checkToken()
    }
    checkTokenOnAccessTokenChange()
  }, [authStore.accessToken]);

  return (
    <MenuProvider>
        <SafeAreaView style={{height: '100%', backgroundColor: colors.red}}>
        <StatusBar barStyle='light-content' backgroundColor={colors.red}/>
        <NavigationContainer>  
          {checkingToken ? <Splash/> :
          <QueryClientProvider client={queryClient}>
            {authStore.accessToken ? <AuthorizedNavigator  /> : <UnauthorizedNavigator /> } 
          </QueryClientProvider> }
        </NavigationContainer>  
      </SafeAreaView>
    </MenuProvider>
  );
}

export default observer(Atlas);